﻿using Infragistics.Win.UltraWinGrid;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace WG_SyncMultipleGrids
{
    public partial class Form1 : Form
    {
        private bool colPosChangedInProgress = false;
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            this.SetUpGridDataSource();
        }

        private void SetUpGridDataSource()
        {
            DataSet ds = new DataSet();
            DataTable t1 = ds.Tables.Add("table1");
            t1.Columns.Add("id", typeof(int)).AutoIncrement = true;
            t1.Columns.Add("Name", typeof(string));
            t1.Columns.Add("Quantity", typeof(int));
            t1.Columns.Add("Price", typeof(double));
            t1.Columns.Add("Date", typeof(DateTime));

            ds.Tables["table1"].Rows.Add(new object[] { null, "Beverage", 50, 10.2, new DateTime(2008, 04, 22) });
            ds.Tables["table1"].Rows.Add(new object[] { null, "Appliances", 5, 90, new DateTime(2012, 05, 10) });
            ds.Tables["table1"].Rows.Add(new object[] { null, "FastFood", 20, 32, new DateTime(2005, 10, 05) });
            ds.Tables["table1"].Rows.Add(new object[] { null, "Furniture", 12, 79, new DateTime(2009, 02, 28) });
            ds.Tables["table1"].Rows.Add(new object[] { null, "Meat", 40, 10.2, new DateTime(2008, 04, 22) });
            ds.Tables["table1"].Rows.Add(new object[] { null, "Fruits", 25, 12, new DateTime(2012, 05, 10) });
            ds.Tables["table1"].Rows.Add(new object[] { null, "Vegetables", 30, 13, new DateTime(2005, 10, 05) });
            ds.Tables["table1"].Rows.Add(new object[] { null, "Cars", 13, 200, new DateTime(2009, 02, 28) });
            ds.Tables["table1"].Rows.Add(new object[] { null, "Tools", 60, 15.3, new DateTime(2008, 04, 22) });

            ultraGrid1.DataSource = ds;
            ultraGrid2.DataSource = ds;
            ultraGrid3.DataSource = ds;
        }

        private void ultraGrid_ColPosChanged(object sender, BeforeColPosChangedEventArgs e)
        {
            if (colPosChangedInProgress)
                return;

            var gridsToModify = new List<UltraGrid>();

            gridsToModify.AddRange(new UltraGrid[] { ultraGrid1, ultraGrid2, ultraGrid3 });

            gridsToModify.Remove((UltraGrid)sender);

            try
            {
                colPosChangedInProgress = true;

                Action<UltraGridColumn, Infragistics.Win.UltraWinGrid.ColumnHeader> actionToPerform = null;

                switch (e.PosChanged)
                {
                    case PosChanged.Moved:
                        actionToPerform = new Action<UltraGridColumn, Infragistics.Win.UltraWinGrid.ColumnHeader>((c, h) => c.Header.VisiblePosition = h.VisiblePosition);
                        break;
                    case PosChanged.Swapped:
                        actionToPerform = new Action<UltraGridColumn, Infragistics.Win.UltraWinGrid.ColumnHeader>((c, h) => c.Header.VisiblePosition = h.VisiblePosition);
                        break;
                    case PosChanged.Sized:
                        actionToPerform = new Action<UltraGridColumn, Infragistics.Win.UltraWinGrid.ColumnHeader>( (c, h) => c.Width =h.Column.Width);
                        break;
                    case PosChanged.HiddenStateChanged:
                        actionToPerform = new Action<UltraGridColumn, Infragistics.Win.UltraWinGrid.ColumnHeader>((c, h) => c.Hidden = h.Column.Hidden);
                        break;
                    default:
                        Debug.Fail("Unexpected value for PosChanged.");
                        break;
                }

                this.ModifyColumns(e.ColumnHeaders, gridsToModify, actionToPerform);

            }
            finally
            {
                colPosChangedInProgress = false;                                                                                                                             
            }
        }

        private void ModifyColumns(Infragistics.Win.UltraWinGrid.ColumnHeader[] columnHeaders, List<UltraGrid> gridsToModify, Action<UltraGridColumn, Infragistics.Win.UltraWinGrid.ColumnHeader> actionToPerform)
        {
            if (columnHeaders.Length == 0)
                return;

            var band = columnHeaders[0].Band;

            foreach (var gridToModify in gridsToModify)
            {
               var bandToModify = gridToModify.DisplayLayout.Bands[band.Key];

                foreach (var column in bandToModify.Columns)
                {
                    var modifiedHeader = columnHeaders.FirstOrDefault(c => c.Caption == column.Header.Caption);
                    if (modifiedHeader != null)
                    {
                        actionToPerform(column, modifiedHeader);
                    }
                }
            }
        }

        private void ultraGrid_InitializeLayout(object sender, InitializeLayoutEventArgs e)
        {
            e.Layout.AutoFitStyle = AutoFitStyle.ResizeAllColumns;
        }
    }
}
